import "Turbine";
import "Turbine.UI";
import "PengorosPlugins.BuffBars";

Icone = class( Turbine.UI.Window );

ix = Turbine.UI.Display.GetWidth()/2-45;
iy = Turbine.UI.Display.GetHeight()/2-45;

function Icone:Constructor()
	
	IconeCharge();
	
	Turbine.UI.Window.Constructor( self );
	
	self.locked = PengorosPlugins.Utils.KeyManager.IsLocked();
    self.hidden = PengorosPlugins.Utils.KeyManager.IsHidden();

	
	self:SetPosition( ix, iy);
	self:SetSize(45,45);
	self:SetBackColor(Turbine.UI.Color(0,0,1,0));
	self:SetVisible( true );
	
	self.button = Turbine.UI.Control();
	self.button:SetParent(self);
	self.button:SetPosition(5,5);
	self.button:SetSize(35,35);
	self.button:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	if (flagVisible == true) then
		self.button:SetBackground("/PengorosPlugins/UI/Resources/Icon_On.tga");
	else
		self.button:SetBackground("/PengorosPlugins/UI/Resources/Icon_Off.tga");
	end
	self:SetOpacity(0.5);
	self.button.MouseEnter = function(sender,args)
		self:SetOpacity(1);
	end
	self.button.MouseLeave = function(sender,args)
		self:SetOpacity(0.5);
	end		
	
	
    -- hook effect events
    PengorosPlugins.Utils.KeyManager.AddListener(self);
    PengorosPlugins.Utils.EffectManager.AddListener(self);
    PengorosPlugins.Utils.CoolDownManager.AddListener(self);
    
	
    -- add a context menu to show the options
    local contextMenu = Turbine.UI.ContextMenu();
    local menuItems = contextMenu:GetItems();
    menuItems:Add(Turbine.UI.MenuItem(L["Options"]));

    menuItems:Get(1).Click = function(sender, args)
    	Turbine.PluginManager.ShowOptions(Plugins.BuffBars);
    end;
    self.button.MouseClick = function(sender, args)
        if (args.Button == Turbine.UI.MouseButton.Right) then
            contextMenu:ShowMenu();
		elseif(args.Button == Turbine.UI.MouseButton.Left) then
			if (flagVisible == true) then
				self.button:SetBackground("/PengorosPlugins/UI/Resources/Icon_Off.tga");
				flagVisible=false;
			else
				self.button:SetBackground("/PengorosPlugins/UI/Resources/Icon_On.tga");
				flagVisible=true;
			end

            self:UpdateUIElements(flagVisible);
        end

    end
		
end

function Icone:UpdateUIElements(flagVisible)

    for i = 1, table.getn(effectWindows) do
        effectWindows[i]:SetVisible(flagVisible);
    end

    for i = 1, table.getn(effectSliders) do
        effectSliders[i]:UpdateVisibility();
    end

    for i = 1, table.getn(quickslotBars) do
        quickslotBars[i]:SetVisible(flagVisible);
    end

end

function Icone:UpdateVisibility()
    local visible;
    if (self.hidden) then
        visible = false;
    else
        visible = true;
    end
    self:SetVisible(visible);
end

function Icone:InCombatChanged(inCombat)
    self:UpdateVisibility();
end

function Icone:MountChanged(mounted, combat)
    self:UpdateVisibility();
end

function Icone:UiLocked(locked)
    self.locked = locked;
    if (not self.locked) then

        self.dragBar = PengorosPlugins.UI.DragBar(self, "BuffBars: ".."Icone");
        self.dragBar:SetBarOnTop(true);
        self.dragBar:SetBarVisible(not self.hidden);
        self.dragBar:SetDraggable(true);
        
        self:UpdateVisibility();
    else
        if (self.dragBar ~= nil) then
            self.dragBar:SetDraggable(false);
            self.dragBar:SetTarget(nil);
            self.dragBar = nil;
        end
        ix = self:GetLeft();
		iy = self:GetTop();
        self:SetBackColor(Turbine.UI.Color(0, 0, 0, 0));
    end
end

function Icone:UiHidden(hidden)
    self.hidden = hidden;
    if (self.dragBar ~= nil) then
        self.dragBar:SetVisible(not hidden);
    end
    self:UpdateVisibility();
end


function IconeSauve()
	settings={};
	settings.Top=iy;
	settings.Left=ix;
	settings.Flag=flagVisible;
	Turbine.PluginData.Save(Turbine.DataScope.Account, "BuffBarsIcone", settings);
end

function IconeCharge()
	settings=Turbine.PluginData.Load(Turbine.DataScope.Account, "BuffBarsIcone");
	if settings==nil then
		settings={};
	else
		ix = settings.Left;
		iy = settings.Top;
		flagVisible = settings.Flag;
	end
end

Icone();